@@ -2,6 +2,7 @@
 #define MATRIX_MUTILS_H
 
 #include <Rdefines.h>
+#include <Rconfig.h>
 
 char norm_type(char *typstr);
 char rcond_type(char *typstr);
@@ -38,6 +39,7 @@ extern SEXP
     Matrix_DIsqrtSym,
     Matrix_DimSym,
     Matrix_GpSym,
+    Matrix_LSym,
     Matrix_LiSym,
     Matrix_LpSym,
     Matrix_LxSym,
@@ -45,6 +47,7 @@ extern SEXP
     Matrix_ParentSym,
     Matrix_RXXSym,
     Matrix_RZXSym,
+    Matrix_TSym,
     Matrix_XtXSym,
     Matrix_ZtXSym,
     Matrix_ZZxSym,
